package com.sun.corba.se.impl.naming.cosnaming;

import org.omg.CosNaming.NamingContextHelper;
import org.omg.CosNaming.NamingContext;
import org.omg.CosNaming.BindingIteratorHelper;
import org.omg.CosNaming.BindingIteratorHolder;
import org.omg.CosNaming.BindingListHolder;
import org.omg.CosNaming.BindingTypeHolder;
import org.omg.CORBA.SystemException;
import java.util.logging.Level;
import org.omg.CosNaming.Binding;
import org.omg.CosNaming.BindingType;
import org.omg.CosNaming.NameComponent;
import org.omg.PortableServer.POA;
import com.sun.corba.se.spi.orb.ORB;
import org.omg.CORBA.Object;
import java.util.Hashtable;
import com.sun.corba.se.impl.logging.NamingSystemException;
import java.util.logging.Logger;

public class TransientNamingContext extends NamingContextImpl implements NamingContextDataStore
{
    private Logger readLogger;
    private Logger updateLogger;
    private Logger lifecycleLogger;
    private NamingSystemException wrapper;
    private final Hashtable theHashtable;
    public org.omg.CORBA.Object localRoot;
    
    public TransientNamingContext(ORB orb, org.omg.CORBA.Object localRoot, POA poa) throws Exception {
        super(orb, poa);
        this.theHashtable = new Hashtable();
        this.wrapper = NamingSystemException.get(orb, "naming");
        this.localRoot = localRoot;
        this.readLogger = orb.getLogger("naming.read");
        this.updateLogger = orb.getLogger("naming.update");
        (this.lifecycleLogger = orb.getLogger("naming.lifecycle")).fine("Root TransientNamingContext LIFECYCLE.CREATED");
    }
    
    @Override
    public final void Bind(NameComponent nameComponent, org.omg.CORBA.Object theObjectRef, BindingType bindingType) throws SystemException {
        InternalBindingKey key = new InternalBindingKey(nameComponent);
        InternalBindingValue value = new InternalBindingValue(new Binding(new NameComponent[] { nameComponent }, bindingType), null);
        value.theObjectRef = theObjectRef;
        if ((InternalBindingValue)this.theHashtable.put(key, value) != null) {
            this.updateLogger.warning("<<NAMING BIND>>Name " + this.getName(nameComponent) + " Was Already Bound");
            throw this.wrapper.transNcBindAlreadyBound();
        }
        if (this.updateLogger.isLoggable(Level.FINE)) {
            this.updateLogger.fine("<<NAMING BIND>><<SUCCESS>>Name Component: " + nameComponent.id + "." + nameComponent.kind);
        }
        return;
    }
    
    @Override
    public final org.omg.CORBA.Object Resolve(NameComponent nameComponent, BindingTypeHolder bindingTypeHolder) throws SystemException {
        if (nameComponent.id.length() == 0 && nameComponent.kind.length() == 0) {
            bindingTypeHolder.value = BindingType.ncontext;
            return this.localRoot;
        }
        InternalBindingValue internalBindingValue = (InternalBindingValue)this.theHashtable.get(new InternalBindingKey(nameComponent));
        if (internalBindingValue == null) {
            return null;
        }
        if (this.readLogger.isLoggable(Level.FINE)) {
            this.readLogger.fine("<<NAMING RESOLVE>><<SUCCESS>>Namecomponent :" + this.getName(nameComponent));
        }
        bindingTypeHolder.value = internalBindingValue.theBinding.binding_type;
        return internalBindingValue.theObjectRef;
    }
    
    @Override
    public final org.omg.CORBA.Object Unbind(NameComponent nameComponent) throws SystemException {
        InternalBindingValue internalBindingValue = (InternalBindingValue)this.theHashtable.remove(new InternalBindingKey(nameComponent));
        if (internalBindingValue == null) {
            if (this.updateLogger.isLoggable(Level.FINE)) {
                this.updateLogger.fine("<<NAMING UNBIND>><<FAILURE>> There was no binding with the name " + this.getName(nameComponent) + " to Unbind ");
            }
            return null;
        }
        else {
            if (this.updateLogger.isLoggable(Level.FINE)) {
                this.updateLogger.fine("<<NAMING UNBIND>><<SUCCESS>> NameComponent:  " + this.getName(nameComponent));
            }
            return internalBindingValue.theObjectRef;
        }
    }
    
    @Override
    public final void List(int n, BindingListHolder bindingListHolder, BindingIteratorHolder bindingIteratorHolder) throws SystemException {
        try {
            TransientBindingIterator transientBindingIterator = new TransientBindingIterator(this.orb, (Hashtable)this.theHashtable.clone(), this.nsPOA);
            transientBindingIterator.list(n, bindingListHolder);
            bindingIteratorHolder.value = BindingIteratorHelper.narrow(this.nsPOA.id_to_reference(this.nsPOA.activate_object(transientBindingIterator)));
        }
        catch (SystemException obj) {
            this.readLogger.warning("<<NAMING LIST>><<FAILURE>>" + obj);
            throw obj;
        }
        catch (Exception obj2) {
            this.readLogger.severe("<<NAMING LIST>><<FAILURE>>" + obj2);
            throw this.wrapper.transNcListGotExc(obj2);
        }
    }
    
    @Override
    public final NamingContext NewContext() throws SystemException {
        try {
            org.omg.CORBA.Object id_to_reference = this.nsPOA.id_to_reference(this.nsPOA.activate_object(new TransientNamingContext(this.orb, this.localRoot, this.nsPOA)));
            this.lifecycleLogger.fine("TransientNamingContext LIFECYCLE.CREATE SUCCESSFUL");
            return NamingContextHelper.narrow(id_to_reference);
        }
        catch (SystemException thrown) {
            this.lifecycleLogger.log(Level.WARNING, "<<LIFECYCLE CREATE>><<FAILURE>>", thrown);
            throw thrown;
        }
        catch (Exception thrown2) {
            this.lifecycleLogger.log(Level.WARNING, "<<LIFECYCLE CREATE>><<FAILURE>>", thrown2);
            throw this.wrapper.transNcNewctxGotExc(thrown2);
        }
    }
    
    @Override
    public final void Destroy() throws SystemException {
        try {
            byte[] servant_to_id = this.nsPOA.servant_to_id(this);
            if (servant_to_id != null) {
                this.nsPOA.deactivate_object(servant_to_id);
            }
            if (this.lifecycleLogger.isLoggable(Level.FINE)) {
                this.lifecycleLogger.fine("<<LIFECYCLE DESTROY>><<SUCCESS>>");
            }
        }
        catch (SystemException thrown) {
            this.lifecycleLogger.log(Level.WARNING, "<<LIFECYCLE DESTROY>><<FAILURE>>", thrown);
            throw thrown;
        }
        catch (Exception thrown2) {
            this.lifecycleLogger.log(Level.WARNING, "<<LIFECYCLE DESTROY>><<FAILURE>>", thrown2);
            throw this.wrapper.transNcDestroyGotExc(thrown2);
        }
    }
    
    private String getName(NameComponent nameComponent) {
        return nameComponent.id + "." + nameComponent.kind;
    }
    
    @Override
    public final boolean IsEmpty() {
        return this.theHashtable.isEmpty();
    }
}
